#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define SIGFILE "signok.txt"
#define LOGFILE "pgpfake.log"
#define dprintf if (debug) fprintf

int  debug=0;
char cmdline[256], sigfile[256], logfile[256], line[256];


int convert_signature(char *sigfile)
{
  char dummy[80], answer[6], date[12], name[80];
  FILE *sig;

  /* read result of signature check */
  sig = fopen(sigfile, "r");
  while (!feof(sig))
  {
    fgets(line, 256, sig);
    if (strstr(line,"signature") != NULL)
    {
      sscanf(line, "%s", answer);
      sscanf(line, "%[^0-9]%s", dummy, date);

      fgets(line, 256, sig);   /* read dummy line */
      fgets(line, 256, sig);   /* read name */
      sscanf(line, " \"%[^\"]", name);
      if (stricmp(answer,"GOOD") == 0)
        fprintf(stderr, "Good signature from user \"%s\".\nSignature made %s\n", name, date);
      else
        fprintf(stderr, "Bad signature from user \"%s\".\nSignature made %s\n", name, date);
      break;
    }
  }
  fclose(sig);
  remove(sigfile);
  return (0);
}


int pgp_check_signature(FILE *log, char *pgppath, char *sigfile)
{
  int i, rc;
  FILE *pgpcmd;

  rc = 0;
  sprintf(cmdline, "%s\\pgp5.exe v -q -f 2>%s", pgppath, sigfile);
  pgpcmd = popen(cmdline, "w");
  if (pgpcmd == NULL)
  {
    dprintf(log, "\npopen failed with %d!\n", errno);
    return(-1);
  }

  i=0;
  while (!feof(stdin))
  {
    fgets(line, 256, stdin);
    fputs(line, pgpcmd);
    i++;
  }
  dprintf(log,"\n%d lines processed\n",i-1);
  rc = pclose(pgpcmd);

  convert_signature(sigfile);
  return(rc);
}


int pgp_encrypt(FILE *log, char *pgppath, char *to)
{
  int i, rc;
  FILE *pgpcmd;

  rc = 0;
  sprintf(cmdline, "%s\\pgp5.exe e -f -a -q -r %s", pgppath, to);
  pgpcmd = popen(cmdline, "w");
  if (pgpcmd == NULL)
  {
    dprintf(log, "\npopen failed with %d!\n", errno);
    return(-1);
  }
  i=0;
  while (!feof(stdin))
  {
    fgets(line, 256, stdin);
    fputs(line, pgpcmd);
    i++;
  }
  dprintf(log,"\n%d lines processed\n",i-1);
  rc = pclose(pgpcmd);

  return(rc);
}


int pgp_sign(FILE *log, char *pgppath, char *passphrase, char *userid)
{
  int i, rc;
  FILE *pgpcmd;

  rc = 0;
  sprintf(cmdline, "%s\\pgp5.exe s -f -a -q -b -z\"%s\" -u %s", pgppath, passphrase, userid);
  pgpcmd = popen(cmdline, "w");
  if (pgpcmd == NULL)
  {
    dprintf(log, "\npopen failed with %d!\n", errno);
    return(-1);
  }
  i=0;
  while (!feof(stdin))
  {
    fgets(line, 256, stdin);
    fputs(line, pgpcmd);
    i++;
  }
  dprintf(log,"\n%d lines processed\n",i-1);
  rc = pclose(pgpcmd);

  return(rc);
}


int pgp_decrypt(FILE *log, char *pgppath, char *sigfile, char *passphrase)
{
  int i, rc;
  FILE *pgpcmd;

  rc = 0;
  sprintf(cmdline, "%s\\pgp5.exe v -f -q -z\"%s\" 2>%s", pgppath, passphrase, sigfile);
  pgpcmd = popen(cmdline, "w");
  if (pgpcmd == NULL)
  {
    dprintf(log, "\npopen failed with %d!\n", errno);
    return(-1);
  }
  i=0;
  while (!feof(stdin))
  {
    fgets(line, 256, stdin);
    fputs(line, pgpcmd);
    i++;
  }
  dprintf(log,"\n%d lines processed\n",i-1);
  rc = pclose(pgpcmd);

  convert_signature(sigfile);
  return(rc);
}


int pgp_extract_my_key(char *pgppath, char *userid)
{
  int rc;

  sprintf(cmdline, "%s\\pgpk.exe -xa %s", pgppath, userid);
  rc = system(cmdline);

  return(0);
}


int pgp_add_new_key(FILE *log, char *pgppath)
{
  int  i, rc;
  FILE *pgpcmd;

  rc = 0;
  sprintf(cmdline, "%s\\pgp5.exe v -f -q", pgppath);

  pgpcmd = popen(cmdline, "w");
  if (pgpcmd == NULL)
  {
    dprintf(log, "\npopen failed with %d!\n", errno);
    return(-1);
  }
  i=0;
  while (!feof(stdin))
  {
    fgets(line, 256, stdin);
    fputs(line, pgpcmd);
    i++;
  }
  dprintf(log,"\n%d lines processed\n",i-1);
  rc = pclose(pgpcmd);

  return(rc);
}


int main(int argc, char *argv[])
{
  int i, rc;
  FILE *log;
  char *pgppath, *tmppath, *debugstr;

  rc = 0;
  tmppath = getenv("TMP");
  if (tmppath == NULL)
  {
    tmppath=".";
  }
  sprintf(sigfile,"%s\\%s",tmppath,SIGFILE);
  sprintf(logfile,"%s\\%s",tmppath,LOGFILE);

  debugstr = getenv("PGPFAKE_DEBUG");
  if (debugstr != NULL)
  {
    debug = 1;
    log = fopen(logfile,"w");
    for (i=1; i<argc; i++)
      fprintf(log, "%d: %s\n",i,argv[i]);
  }

  pgppath = getenv("PGPPATH");
  if (pgppath == NULL)
  {
    dprintf(log, "\nPGP not installed correctly!\n");
    if (debug) fclose(log);
    return(-1);
  }

  switch (argc)
  {
    case 9:  /* called to verify signature */
             dprintf(log, "check signature\n");
             rc = pgp_check_signature(log, pgppath, sigfile);
             break;

    case 11: /* called to extract public key */
             /*
                argv[10]: my userid
             */
             dprintf(log, "extract key\n");
             rc = pgp_extract_my_key(pgppath, argv[10]);
             break;

    case 12: /* called to decrypt or to add key information */
             /*
                decrypt: argv[9] -> passphrase
                addkey:  argv[9] = "-a"
             */
             if (argv[9][1] == 'a') /* called to add public key */
             {
               dprintf(log, "addkey\n");
               /* rc = pgp_add_new_key(log, pgppath); */
               /* PGP asks for confirmation...        */
             }
             else                   /* called to decrypt */
             {
               dprintf(log, "decrypt\n");
               rc = pgp_decrypt(log, pgppath, sigfile, argv[9]+2);
             }
             break;

    case 13: /* called to encrypt */
             /*
                argv[11]: userid to encrypt to
                argv[12]: my userid
             */
             dprintf(log, "encrypt\n");
             rc = pgp_encrypt(log, pgppath, argv[11]);
             break;

    case 14: /* called to sign */
             /*
                argv[11]: passphrase
                argv[13]: my userid
             */
             dprintf(log, "sign\n");
             rc = pgp_sign(log, pgppath, argv[11]+2, argv[13]);
             break;

    default: /* unknown call */
             dprintf(log, "unknown call\n");
             break;
  }

  dprintf(log,"\nreturn code: %d\n",rc);
  if (debug) fclose(log);
  return(rc);
}